@@ -33,15 +33,14 @@ const DB_DIR: &str = "db";
 /// `KvStore` is a store of keys and values into a Sled db, while maintaining a maximum disk
 /// usage to restrict storage.
 #[derive(Clone, Debug)]
-pub(crate) struct KvStore<K, V> {
+pub(crate) struct KvStore<V> {
     // tracks space used.
     used_space: UsedSpace,
     db: Db,
-    _k: PhantomData<K>,
     _v: PhantomData<V>,
 }
 
-impl<K: Key, V: Value> KvStore<K, V>
+impl<V: Value> KvStore<V>
 where
     Self: Subdir,
 {
@@ -65,13 +64,12 @@ where
         Ok(KvStore {
             used_space,
             db,
-            _k: PhantomData,
             _v: PhantomData,
         })
     }
 }
 
-impl<K: Key, V: Value + Send + Sync> KvStore<K, V> {
+impl<V: Value + Send + Sync> KvStore<V> {
     ///
     pub(crate) async fn total_used_space(&self) -> u64 {
         self.used_space.total().await
@@ -97,96 +95,39 @@ impl<K: Key, V: Value + Send + Sync> KvStore<K, V> {
     /// If there is not enough storage space available, returns `Error::NotEnoughSpace`.  In case of
     /// an IO error, it returns `Error::Io`.
     ///
-    /// If a value with the same id already exists, it will be overwritten.
+    /// If a value with the same id already exists, it will not be overwritten.
     pub(crate) async fn store(&self, value: &V) -> Result<()> {
         debug!("Writing value to KV store");
 
         let key = value.key().to_db_key()?;
-        let exists = self.db.contains_key(key.clone()).map_err(Error::from)?;
-
-        trace!(">> Does this entry exist; {:?}: {:?}", key, exists);
-
         let serialised_value = serialise(value)?.to_vec();
-        // FIXME: We're not considering overwriting here. So the space isn't necessarily all 'consumed'
-        // if we overwrite same value eg, or it's only 5 bytes longer/shorter etc
-        let consumed_space = serialised_value.len() as u64;
 
-        info!("consumed space: {:?}", consumed_space);
-        if !self.used_space.can_consume(consumed_space).await {
+        let exists = self.db.contains_key(key.clone()).map_err(Error::from)?;
+        if !exists
+            && !self
+                .used_space
+                .can_consume(serialised_value.len() as u64)
+                .await
+        {
             return Err(Error::NotEnoughSpace);
         }
 
-        let res = self.db.insert(key, serialised_value);
-
-        match res {
-            Ok(_) => {
-                info!("Writing value succeeded!");
-                Ok(())
-            }
-            Err(e) => {
-                info!("Writing value failed!");
-                Err(Error::Sled(e))
+        // Atomically write the value if it's new - this prevents multiple concurrent writes from
+        // consuming extra space (since sled is backed by a log).
+        match self
+            .db
+            .compare_and_swap::<_, &[u8], _>(key, None, Some(serialised_value))?
+        {
+            Ok(()) => debug!("Successfully wrote new value"),
+            Err(sled::CompareAndSwapError { .. }) => {
+                // We throw away the value if the compare_and_swap failed since current use-cases do
+                // not require updates. In future we may want to return the preexisting value, or
+                // have a separate API for overwriting stores.
+                debug!("Value already existed, so we didn't have to write")
             }
         }
-    }
 
-    /// Stores a batch of values.
-    ///
-    /// If there is not enough storage space available, returns `Error::NotEnoughSpace`.  In case of
-    /// an IO error, it returns `Error::Io`.
-    ///
-    /// If a value with the same id already exists, it will be overwritten.
-    #[allow(unused)] // this will soon be used
-    pub(crate) async fn store_batch(&self, values: &[V]) -> Result<()> {
-        info!("Writing batch");
-        use rayon::prelude::*;
-        type KvPair = (String, Vec<u8>);
-        type KvPairResults = Vec<Result<KvPair>>;
-
-        let mut batch = sled::Batch::default();
-        let (ok, err_results): (KvPairResults, KvPairResults) = values
-            .par_iter()
-            .map(|value| {
-                let serialised_value = serialise(value)?.to_vec();
-                let key = value.key().to_db_key()?;
-                Ok((key, serialised_value))
-            })
-            .partition(|r| r.is_ok());
-
-        if !err_results.is_empty() {
-            for e in err_results {
-                error!("{:?}", e);
-            }
-
-            return Err(Error::SledBatching);
-        }
-
-        let consumed_space = ok
-            .into_iter()
-            .flatten()
-            .map(|(key, value)| {
-                let consumed_space = value.len() as u64;
-                batch.insert(key.as_bytes(), value);
-                consumed_space
-            })
-            .sum();
-
-        if !self.used_space.can_consume(consumed_space).await {
-            return Err(Error::NotEnoughSpace);
-        }
-
-        let res = self.db.apply_batch(batch);
-
-        match res {
-            Ok(_) => {
-                info!("Writing batch succeeded!");
-                Ok(())
-            }
-            Err(e) => {
-                warn!("Writing batch failed!");
-                Err(Error::Sled(e))
-            }
-        }
+        Ok(())
     }
 
     /// Returns a value previously stored under `key`.
@@ -222,7 +163,6 @@ impl<K: Key, V: Value + Send + Sync> KvStore<K, V> {
     }
 
     /// Lists all keys of currently stored data.
-    #[cfg_attr(not(test), allow(unused))]
     pub(crate) fn keys(&self) -> Result<Vec<V::Key>> {
         let keys = self
             .db
@@ -234,6 +174,14 @@ impl<K: Key, V: Value + Send + Sync> KvStore<K, V> {
             .collect();
         Ok(keys)
     }
+
+    // We should avoid manually flushing in real code, since it can take a lot of time - tuning
+    // `flush_every_ms` would be preferrable. This is useful in tests, however.
+    #[cfg(test)]
+    pub(crate) async fn flush(&self) -> Result<()> {
+        let _bytes_flushed = self.db.flush_async().await?;
+        Ok(())
+    }
 }
 
 pub(crate) fn convert<T: DeserializeOwned>(key: sled::IVec) -> Result<T> {
